﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Bumper : MonoBehaviour
{
    public Transform Cylinder = null;
    public float Angle = 0.0f;
    public float Bounciness = 5.0f;
    public float MaxSpeed = 50.0f;

    public Transform targetBumperPos = null;
    public Transform BumperCylinder = null;
    public Vector3 bumperCylinderStartPos = Vector3.zero;
    public Vector3 diff = Vector3.zero;

    bool hiding = false;
    bool showing = false;
    bool spawning = false;

    public float animTime = 0.0f;
    public float animTimeTotal = 1.0f;

    public float spawningTime = 0.0f;
    public float spawningTimeTotal = 1.0f;

    public Transform SpawnPoint = null;
    public GameObject BallPrefab = null;

    public AudioSource AudioSourceHit = null;
    public AudioSource AudioSourceSpawn = null;

    void Start ()
    {
        bumperCylinderStartPos = BumperCylinder.transform.position;
        diff = targetBumperPos.position - bumperCylinderStartPos;
    }
	
	void Update ()
    {
        if (hiding || showing)
        {
            animTime -= Time.deltaTime;
            if (animTime > 0.0f)
            {
                float value = Mathf.Sin(1.0f - (animTime / animTimeTotal));

                if (showing)
                {
                    value = 1.0f - value;
                }

                BumperCylinder.transform.position = bumperCylinderStartPos + (value * diff);
            }
            else
            {
                if (hiding)
                {                    
                    hiding = false;
                    animTime = animTimeTotal;
                    spawning = true;
                    spawningTime = spawningTimeTotal;
                    Instantiate(BallPrefab, SpawnPoint.position, SpawnPoint.rotation);
                    AudioSourceSpawn.Play();
                }
                else if (showing)
                {
                    showing = false;
                }
            }

        }
        else if (spawning)
        {
            spawningTime -= Time.deltaTime;
            if (spawningTime <= 0.0f)
            {
                showing = true;
                spawning = false;
            }
        }
	}

    private void OnCollisionEnter(Collision collision)
    {
        if (!hiding && !showing && !spawning)
        {
            var ball = collision.collider.gameObject.GetComponent<Ball>();
            if (ball != null)
            {
                Hit();
                ball.Rigidbody.velocity *= Bounciness;

                if (ball.Rigidbody.velocity.magnitude > MaxSpeed)
                {
                    ball.Rigidbody.velocity = ball.Rigidbody.velocity.normalized * MaxSpeed;
                }
            }
        }
    }

    public void Hit()
    {
        Angle -= 45.0f;
        Cylinder.rotation = Quaternion.Euler(0.0f, 0.0f, Angle);
        if(Angle <= -355.0f)
        {
            Action();
            Angle = 0.0f;
            AudioSourceHit.pitch = 2.25f;
            AudioSourceHit.Play();
        }
        else
        {
            if(Angle > -46.0f)
            {
                AudioSourceHit.pitch = 1.0f;
            }
            else if (Angle > -91.0f)
            {
                AudioSourceHit.pitch = 1.25f;
            }
            else if (Angle > -136.0f)
            {
                AudioSourceHit.pitch = 1.42f;
            }
            else if (Angle > -181.0f)
            {
                AudioSourceHit.pitch = 1.5f;
            }
            else if (Angle > -226.0f)
            {
                AudioSourceHit.pitch = 1.58f;
            }
            else if (Angle > -271.0f)
            {
                AudioSourceHit.pitch = 1.83f;
            }
            else if (Angle > -316.0f)
            {
                AudioSourceHit.pitch = 2.0f;
            }

            AudioSourceHit.Play();
        }
    }

    public void Action()
    {
        hiding = true;
        animTime = animTimeTotal;
    }
}
 