﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class LevelCmparer : IComparer
{
    // Call CaseInsensitiveComparer.Compare with the parameters reversed.
    public int Compare(object x, object y)
    {
        return ((LevelDefinition)x).levelNumber > ((LevelDefinition)y).levelNumber ? 1 : (((LevelDefinition)x).levelNumber < ((LevelDefinition)y).levelNumber ? -1 : 0);
    }
}

public delegate void ControlPanelVisibilityDelegate();

public class ControlPanelScript : MonoBehaviour
{

    bool CreditsVisible = false;

    public TickScript HintsEnabledScript;
    public TickScript FlipperEnabledScript;
    LevelDefinition[] LevelDefinitions;
	public UnityEngine.UI.Dropdown ScreenConfigSelector;
	public UnityEngine.UI.Dropdown LevelSelector;
    public UnityEngine.UI.Slider EveryPuzzleSlider;
    public UnityEngine.UI.Text EveryPuzzleLabel;
    public UnityEngine.UI.Slider PuzzleTimeSlider;
    public UnityEngine.UI.Text PuzzleTimeLabel;
    public GameObject SelectorLabel;
    public GameManager GameMgr;
    public GameObject CredPanel;
	public Canvas ControlPanelCanvas;
	public Button StartButton;
	public Button ExitButton;


	public static ControlPanelScript Instance = null;

    public bool IsHintEnabled = true;

	public event ControlPanelVisibilityDelegate OnControlPanelHidden;

    private RunConfiguration CurrentRunConfig = new RunConfiguration();

    void Awake()
    {
        Instance = this;
    }

    public void TriggerCredits()
    {
        CreditsVisible = !CreditsVisible;

        CredPanel.SetActive(CreditsVisible);
    }

    public void StartPressed()
    {
        GameMgr.CreateNewPuzzle();
		GameMgr.gridManager.OnStartGame();

		StartButton.interactable = false; 

		//hide the panel only if we're in single display mode
		if (AppSettings.Instance.ScreenConfig == EScreenConfig.SingleScreen)
		{
			ControlPanelCanvas.enabled = false;
			if (OnControlPanelHidden != null)
				OnControlPanelHidden();
		}
    }

    public void ResetPressed()
    {
        BallManager.Instance.Reset();
    }



    public RunConfiguration GetCurrentRunConfiguration()
    {
        return CurrentRunConfig;
    }

    void SetFlipperEnabled(bool enabled)
    {
        CurrentRunConfig.ShouldRunFlipper = enabled;
    }

    void SetHintsEnabled(bool enabled)
    {
        IsHintEnabled = enabled;
        Debug.Log(IsHintEnabled);
    }

	public void OnScreenConfigChanged()
	{
		int new_value = ScreenConfigSelector.value;
		AppSettings.Instance.ScreenConfig = (EScreenConfig)new_value;
	}

    public void OnLevelSelectorChanged(int changed)
    {
        string level_name = SelectorLabel.GetComponent<Text>().text;
        foreach (LevelDefinition def in LevelDefinitions)
        {
            if (level_name.Equals(def.levelName))
            {
                CurrentRunConfig.LevelDef = def;
                break;
            }
        }
    }

	public void ExitPressed() 
	{
		Application.Quit ();
	}

    // Use this for initialization
    void Start () {
        CredPanel.SetActive(false);

        FlipperEnabledScript.OnEnableChanged = SetFlipperEnabled;
        HintsEnabledScript.OnEnableChanged = SetHintsEnabled;

        Resources.LoadAll("", typeof(LevelDefinition));
        LevelDefinitions = (LevelDefinition[])Resources.FindObjectsOfTypeAll(typeof(LevelDefinition));
        List<UnityEngine.UI.Dropdown.OptionData> options = new List<UnityEngine.UI.Dropdown.OptionData>();

        IComparer comp = new LevelCmparer();

        System.Array.Sort(LevelDefinitions, comp);

        for (int i = 0; i < LevelDefinitions.Length; i++)
        {
            
            LevelDefinition def = LevelDefinitions[i];
            Debug.Log(def.levelName);
            UnityEngine.UI.Dropdown.OptionData opt_data = new UnityEngine.UI.Dropdown.OptionData(def.levelName);
            options.Add(opt_data);
        }
        LevelSelector.AddOptions(options);
        if (LevelDefinitions.Length > 0)
        {
            CurrentRunConfig.LevelDef = LevelDefinitions[0];
        }

        CurrentRunConfig.ShouldRunFlipper = FlipperEnabledScript.Enabled;
    }
	
	// Update is called once per frame
	void Update () {

		if (Input.GetKey ("escape")) {
			Application.Quit ();
		}

        EveryPuzzleLabel.text = EveryPuzzleSlider.value.ToString();
        CurrentRunConfig.FlipperEveryXPuzzles = (int)EveryPuzzleSlider.value;

        PuzzleTimeLabel.text = PuzzleTimeSlider.value.ToString();
        CurrentRunConfig.FlipperTime = (int)PuzzleTimeSlider.value;
    }
}
