﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GameManager : MonoBehaviour {

    public GridImagesMgr gridManager;
    public PuzzleRandomizer.SPuzzle currentPuzzle;

    public PuzzleRandomizer randomizer;

	public AudioSource AudioSuccess = null;

	public float waitOnSuccess = 3.0f;

    private float timeLeft = 5.0f;

    private int num_success = 0;

    private bool flipper_active = false;

    private float flipper_time = 0.0F;

    // Use this for initialization
    void Start () {
   //     randomizer = new PuzzleRandomizer();
        randomizer.gridManager = gridManager;
		InputAdapter.Instance.OnActionChanged.Add(OnActionsChanged);
        gridManager.OnFadeToGridFinished += UnloadFlipper;
        
		//   CreateNewPuzzle();
	}
	
	// Update is called once per frame
	void Update () {
        if (flipper_active)
        {
            flipper_time += Time.deltaTime;
            if (flipper_time > ControlPanelScript.Instance.GetCurrentRunConfiguration().FlipperTime)
            {
                flipper_time = 0.0F;
                flipper_active = false;
                num_success = 0;

                StopFlipper();

                CreateNewPuzzle();
            }
        }
    }

    bool CheckMatchAndRemove(List<InputAction> actions, int x, int y, bool remove = true)
    {
        if (x == 0 && y == 0)
        {
            if (actions.Contains(InputAction._0))
            {
                if (remove)
                    actions.Remove(InputAction._0);
                return true;
            }
        }
        else if (x == 1 && y == 0)
        {
            if (actions.Contains(InputAction._1))
            {
                if (remove)
                    actions.Remove(InputAction._1);
                return true;
            }
        }
        else if (x == 2 && y == 0)
        {
            if (actions.Contains(InputAction._2))
            {
                if (remove)
                    actions.Remove(InputAction._2);
                return true;
            }
        }
        else if (x == 0 && y == 1)
        {
            if (actions.Contains(InputAction._3))
            {
                if (remove)
                    actions.Remove(InputAction._3);
                return true;
            }
        }
        else if (x == 1 && y == 1)
        {
            if (actions.Contains(InputAction._4))
            {
                if (remove)
                    actions.Remove(InputAction._4);
                return true;
            }
        }
        else if (x == 2 && y == 1)
        {
            if (actions.Contains(InputAction._5))
            {
                if (remove)
                    actions.Remove(InputAction._5);
                return true;
            }
        }
        else if (x == 0 && y == 2)
        {
            if (actions.Contains(InputAction._6))
            {
                if (remove)
                    actions.Remove(InputAction._6);
                return true;
            }
        }
        else if (x == 1 && y == 2)
        {
            if (actions.Contains(InputAction._7))
            {
                if (remove)
                    actions.Remove(InputAction._7);
                return true;
            }
        }
        else if (x == 2 && y == 2)
        {
            if (actions.Contains(InputAction._8))
            {
                if (remove)
                    actions.Remove(InputAction._8);
                return true;
            }
        }
        return false;
    }

    void OnActionsChanged()
    {
        List<InputAction> active_actions = InputAdapter.Instance.GetActiveActions();
        bool success = true;
        SSelectionInfo info = new SSelectionInfo();
        info.grid = new ESelection[3,3];

		if (currentPuzzle == null)
			return;

        for (int x = 0; x < 3; x ++)
        {
            for (int y = 0; y < 3; y++)
            {
                info.grid[y, x] = ESelection.None;
                if (currentPuzzle.cells[x,y].isSolution)
                {
                    if (!CheckMatchAndRemove(active_actions, x, y))
                    {
                        success = false;
                    }
                    else
                    {
                        //good selection
                        info.grid[y, x] = ESelection.Right;
                    }
                }
                else
                {
                    if (CheckMatchAndRemove(active_actions, x, y, false))
                    {
                        //Wrong selection
                        info.grid[y, x] = ESelection.Wrong;
                    }
                }
            }
        }
        gridManager.OnSelectionStateChanged(info);
        if (success)
        {
            if (active_actions.Count == 0)
            {
                OnSuccess();
            }
        }
    }

    void OnSuccess()
    {
		StartCoroutine (Delay (waitOnSuccess));
		AudioSuccess.Play ();
        CreateNewPuzzle();
        num_success++;
        if (num_success >= ControlPanelScript.Instance.GetCurrentRunConfiguration().FlipperEveryXPuzzles)
        {
            if (ControlPanelScript.Instance.GetCurrentRunConfiguration().ShouldRunFlipper)
            {
                num_success = 0;
                StartFlipper();
            }
            
        }
        
    }

	private IEnumerator Delay(float delay)
	{
		Time.timeScale = 0.0f;
		float pauseEndTime = Time.realtimeSinceStartup + delay;
		while (Time.realtimeSinceStartup < pauseEndTime)
		{
			yield return 0;
		}
		Time.timeScale = 1;
	}

    void StartFlipper()
    {
        if (flipper_active)
            return;

        GetComponent<StreamScene>().LoadScene();

        gridManager.PlayTransitionToFlippersAnimation();
        flipper_active = true;
    }

    void StopFlipper()
    {
        gridManager.PlayTransitionToGridAnimation();
    }

    void UnloadFlipper(object sender, System.EventArgs args)
    {
        GetComponent<StreamScene>().UnloadScene();
        Ball.ResetCount();
    }

    public void CreateNewPuzzle()
    {
        InputAdapter.Instance.HackCleanInputs();
        currentPuzzle = randomizer.CreateNewPuzzle();
    }
}
