﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum InputAction
{
    _6, _7, _8,
    _3, _4, _5,
    _0, _1, _2
}



public class InputAdapter : MonoBehaviour {

    public bool IsAsyncInput = true;
    const int NumActions = 9;
    bool[] IsActionPressed = new bool[NumActions];
    bool[] PrevActions = new bool[NumActions];

    bool input_hack = false;

    public delegate void VoidSignal();
    public List<VoidSignal> OnActionChanged = new List<VoidSignal>();

	public static InputAdapter Instance = null;

    public void HackCleanInputs()
    {
        input_hack = true;
    }

	void Awake()
	{
		Instance = this;
	}

    void ResetInputState()
    {
        for (int i = 0; i < NumActions; i++)
        {
            PrevActions[i] = IsActionPressed[i];
            IsActionPressed[i] = false;
        }
    }

    // Use this for initialization
    void Start () {
        ResetInputState();
	}

    public List<InputAction> GetActiveActions()
    {
        List<InputAction> ret = new List<InputAction>();
        if (input_hack)
            return ret;

        for (int i = 0; i < NumActions; i++)
        {
            if(IsActionPressed[i])
            {
                if (i==0)
                {
                    ret.Add(InputAction._0);
                }
                else if (i==1)
                {
                    ret.Add(InputAction._1);
                }
                else if (i == 2)
                {
                    ret.Add(InputAction._2);
                }
                else if (i == 3)
                {
                    ret.Add(InputAction._3);
                }
                else if (i == 4)
                {
                    ret.Add(InputAction._4);
                }
                else if (i == 5)
                {
                    ret.Add(InputAction._5);
                }
                else if (i == 6)
                {
                    ret.Add(InputAction._6);
                }
                else if (i == 7)
                {
                    ret.Add(InputAction._7);
                }
                else if (i == 8)
                {
                    ret.Add(InputAction._8);
                }
            }
        }

        return ret;
    }


    void ReadNumPad()
    {
        if (Input.GetKey(KeyCode.Keypad1))
        {
            ActionPressed(InputAction._0);
        }
        if (Input.GetKey(KeyCode.Keypad2))
        {
            ActionPressed(InputAction._1);
        }
        if (Input.GetKey(KeyCode.Keypad3))
        {
            ActionPressed(InputAction._2);
        }
        if (Input.GetKey(KeyCode.Keypad4))
        {
            ActionPressed(InputAction._3);
        }
        if (Input.GetKey(KeyCode.Keypad5))
        {
            ActionPressed(InputAction._4);
        }
        if (Input.GetKey(KeyCode.Keypad6))
        {
            ActionPressed(InputAction._5);
        }
        if (Input.GetKey(KeyCode.Keypad7))
        {
            ActionPressed(InputAction._6);
        }
        if (Input.GetKey(KeyCode.Keypad8))
        {
            ActionPressed(InputAction._7);
        }
        if (Input.GetKey(KeyCode.Keypad9))
        {
            ActionPressed(InputAction._8);
        }
    }

    void ReadQWE()
    {
        if (Input.GetKey(KeyCode.Z))
        {
            ActionPressed(InputAction._0);
        }
        if (Input.GetKey(KeyCode.X))
        {
            ActionPressed(InputAction._1);
        }
        if (Input.GetKey(KeyCode.C))
        {
            ActionPressed(InputAction._2);
        }
        if (Input.GetKey(KeyCode.A))
        {
            ActionPressed(InputAction._3);
        }
        if (Input.GetKey(KeyCode.S))
        {
            ActionPressed(InputAction._4);
        }
        if (Input.GetKey(KeyCode.D))
        {
            ActionPressed(InputAction._5);
        }
        if (Input.GetKey(KeyCode.Q))
        {
            ActionPressed(InputAction._6);
        }
        if (Input.GetKey(KeyCode.W))
        {
            ActionPressed(InputAction._7);
        }
        if (Input.GetKey(KeyCode.E))
        {
            ActionPressed(InputAction._8);
        }
    }

    void ReadMatt()
    {
        if (Input.GetButton("X"))
        {
            ActionPressed(InputAction._1);
        }
        if (Input.GetButton("W"))
        {
            ActionPressed(InputAction._7);
        }
        if (Input.GetButton("D"))
        {
            ActionPressed(InputAction._5);
        }
        if (Input.GetButton("Z"))
        {
            ActionPressed(InputAction._0);
        }
        if (Input.GetButton("C"))
        {
            ActionPressed(InputAction._2);
        }
        if (Input.GetButton("Q"))
        {
            ActionPressed(InputAction._6);
        }
        if (Input.GetButton("E"))
        {
            ActionPressed(InputAction._8);
        }
        if (Input.GetButton("A"))
        {
            ActionPressed(InputAction._3);
        }
        if (Input.GetAxis("S") > 0)
        {
            ActionPressed(InputAction._4);
        }
    }

    public void SignalIfChanged()
    {
        bool input_changed = false;
        for (int i = 0; i < NumActions; i++)
        {
            if (PrevActions[i] != IsActionPressed[i])
            {
                input_changed = true;
                break;
            }
        }

        if (input_changed)
        {
            input_hack = false;
            for (int i = 0; i < OnActionChanged.Count; i ++)
            {
                OnActionChanged[i]();
            }
        }
    }

	public void ForceInputRefresh()
	{
		for (int i = 0; i < OnActionChanged.Count; i++)
		{
			OnActionChanged[i]();
		}
	}

    // Update is called once per frame
    void Update () {
        
        if (IsAsyncInput)
        {
            ResetInputState();
            ReadNumPad();
            ReadQWE();
            ReadMatt();
            SignalIfChanged();
        }
    }

    public bool IsActionActive(InputAction action)
    {
        switch (action)
        {
            case InputAction._0:
                {
                    return IsActionPressed[0];
                }
            case InputAction._1:
                {
                    return IsActionPressed[1];
                }
            case InputAction._2:
                {
                    return IsActionPressed[2];
                }
            case InputAction._3:
                {
                    return IsActionPressed[3];
                }
            case InputAction._4:
                {
                    return IsActionPressed[4];
                }
            case InputAction._5:
                {
                    return IsActionPressed[5];
                }
            case InputAction._6:
                {
                    return IsActionPressed[6];
                }
            case InputAction._7:
                {
                    return IsActionPressed[7];
                }
            case InputAction._8:
                {
                    return IsActionPressed[8];
                }
        }
        return false;
    }

    public void ActionPressed(InputAction action)
    {
        switch(action)
        {
            case InputAction._0:
                {
                    IsActionPressed[0] = true;
                    break;
                }
            case InputAction._1:
                {
                    IsActionPressed[1] = true;
                    break;
                }
            case InputAction._2:
                {
                    IsActionPressed[2] = true;
                    break;
                }
            case InputAction._3:
                {
                    IsActionPressed[3] = true;
                    break;
                }
            case InputAction._4:
                {
                    IsActionPressed[4] = true;
                    break;
                }
            case InputAction._5:
                {
                    IsActionPressed[5] = true;
                    break;
                }
            case InputAction._6:
                {
                    IsActionPressed[6] = true;
                    break;
                }
            case InputAction._7:
                {
                    IsActionPressed[7] = true;
                    break;
                }
            case InputAction._8:
                {
                    IsActionPressed[8] = true;
                    break;
                }
        }

    }

}
