﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PuzzleRandomizer : MonoBehaviour {
    public GridImagesMgr gridManager;

    private float selectionTimeLeft = 2.0f;
    private float levelTimeLeft = 10.0f;

    public SymbolDefinition[] symbols;

    public Dictionary<Category, List<SymbolDefinition>> symbolMap;

    public SymbolDefinition empty;
    public SymbolDefinition smile;

    public class SPuzzleCell
    {
        public bool isSolution;
        public SymbolDefinition symbol;
    }

    public class SPuzzle
    {
        public SPuzzleCell[,] cells = new SPuzzleCell[3, 3];
    }

    class Coords
    {
        public int x;
        public int y;
        public Coords(int x, int y)
        {
            this.x = x;
            this.y = y;
        }

        public override bool Equals(object obj)
        {
            if (Object.ReferenceEquals(this, obj))
            {
                return true;
            }
            Coords instance = obj as Coords;
            if (instance == null)
            {
                return false;
            }
            return this.x == instance.x && this.y == instance.y;
        }

        public override int GetHashCode()
        {
            return this.x.GetHashCode() ^ this.y.GetHashCode();
        }

        
    }

    class Solutions
    {
        public List<Coords> solutions = new List<Coords>();

        public bool IsSolution(int x, int y)
        {
            return IsSolution(new Coords(x, y));
        }

        public bool IsSolution(Coords cords)
        {
            foreach (var sol in solutions)
            {
                if (sol.Equals(cords))
                    return true;
            }
            return false;
        }
    }




    private void GenerateSolution(Solutions solution)
    {
        var sol = new Coords(Random.Range(0, 3), Random.Range(0, 3));
        while (solution.IsSolution(sol))
        {
            sol.x = Random.Range(0, 3);
            sol.y = Random.Range(0, 3);
        }
        solution.solutions.Add(sol);
    }




    public SPuzzle CreateNewPuzzle()
    {
        var config = ControlPanelScript.Instance.GetCurrentRunConfiguration();
        var level_def = config.LevelDef;
        
        //Choosing solutions
        var sol = new Solutions();
        if (level_def.fieldNum == Fields.Columns)
        {
            var solution = new Coords(0, Random.Range(0, 3));
            sol.solutions.Add(solution);
            solution = new Coords(2, Random.Range(0, 3));
            sol.solutions.Add(solution);
        }
        else
        {
            for (int i = 0; i < level_def.solutionCount; i++)
            {
                GenerateSolution(sol);
            }
        }

        //Choosing images

        PuzzleInfo puzzle_info = new PuzzleInfo();
        puzzle_info.images = new Sprite[3, 3];
        var puzzle = new SPuzzle();
        var cats = System.Enum.GetValues(typeof(Category));
        List<Category> categories = new List<Category>();
        List<Category> unusedCategories = new List<Category>();
   //     int numUsedCategories = 0;
        Debug.Log("Categories:");
        foreach (var cat in cats)
        {
            if (cat.Equals(Category.EMPTY))
                continue;

 //           if (++numUsedCategories > level_def.usedNumOfCategories)
  //              break;

            categories.Add((Category)cat);
            unusedCategories.Add((Category)cat);
            Debug.Log(cat);
        }
        Debug.Log("Categories end");
        Category chosenCategory = (Category)Random.Range(0, categories.Count);
        Debug.Log("Chosen category " + chosenCategory);

        SymbolDefinition firstSolution;
        int solutionIndex = 0;

        //Choose first solution
        solutionIndex = Random.Range(0, symbolMap[chosenCategory].Count);
        firstSolution = symbolMap[chosenCategory][solutionIndex];
        unusedCategories.Remove(chosenCategory);
        Debug.Log("Chosen symbol " + firstSolution.objectName + " " + firstSolution.image);

        List<int> unusedIndices = new List<int>();
        for(int i = 0; i < symbolMap[chosenCategory].Count; i++)
        {
            unusedIndices.Add(i);
        }
        unusedIndices.Remove(solutionIndex);

        for (int y = 0; y < 3; ++y)
        {
            for (int x = 0; x < 3; ++x)
            {
                SPuzzleCell cell = new SPuzzleCell();
                if (sol.IsSolution(x,y))
                {
                    cell.isSolution = true;
                    if(level_def.solutionType == Solution.Same)
                    {
                        cell.symbol = firstSolution;
                    }
                    else
                    {
                        Debug.Log("before " + unusedIndices.Count);
                        var idx = Random.Range(0, unusedIndices.Count);
                        Debug.Log("index " + idx + "category " + chosenCategory);
                        solutionIndex = unusedIndices[idx];
                        unusedIndices.Remove(solutionIndex);
                        Debug.Log("after " + unusedIndices.Count);
                   /*     var nextSymbol = symbolMap[chosenCategory][nextIndex];
                        cell.symbol = nextSymbol;*/

                       // solutionIndex = Random.Range(0, symbolMap[chosenCategory].Count);
                        firstSolution = symbolMap[chosenCategory][solutionIndex];
                        cell.symbol = firstSolution;
                        Debug.Log("Chosen symbol 2 " + firstSolution.objectName + " " + firstSolution.image);
                    }
                }
                else
                {
                    cell.isSolution = false;
                    if (level_def.fieldNum.Equals(Fields.Columns) && x == 1)
                    {
                        cell.symbol = empty;

                    }
                    else
                    {
                        switch (level_def.compliment)
                        {
                            case Compliment.Empty:
                                {
                                    cell.symbol = empty;
                                    break;
                                }
                            case Compliment.DifferentCategory:
                                {
                                    if (unusedCategories.Count > 0)
                                    {
                                        //Debug.Log("before " + unusedCategories.Count);
                                        var num = Random.Range(0, unusedCategories.Count);
                                        //Debug.Log(num);
                                        Category cat = unusedCategories[num];
                                        unusedCategories.Remove(cat);

                                        Debug.Log("other cat " + cat);
                                        //Debug.Log("after " + unusedCategories.Count);
                                        var idx = Random.Range(0, symbolMap[cat].Count);
                                        var nextSymbol = symbolMap[cat][idx];
                                        Debug.Log("other symbol " + nextSymbol);

                                        cell.symbol = nextSymbol;
                                    }
                                    else
                                    {
                                        cell.symbol = empty;
                                    }
                                    break;
                                }
                            case Compliment.SameCategory:
                                {
                                    if (unusedIndices.Count > 0)
                                    {
                                        Debug.Log("before " + unusedIndices.Count);
                                        var idx = Random.Range(0, unusedIndices.Count);
                                        Debug.Log("index " + idx);
                                        var nextIndex = unusedIndices[idx];
                                        unusedIndices.Remove(nextIndex);
                                        Debug.Log("after " + unusedIndices.Count);
                                        var nextSymbol = symbolMap[chosenCategory][nextIndex];
                                        cell.symbol = nextSymbol;
                                    }
                                    else
                                    {
                                        cell.symbol = empty;
                                    }
                                    break;
                                }
                        }
                    }
                    
                }
                puzzle_info.images[x, y] = cell.symbol.image;
                puzzle.cells[x, y] = cell;
            }
        }

        gridManager.DisplayNewPuzzle(puzzle_info);

        return puzzle;
    }



    // Use this for initialization
    void Start () {

        Resources.LoadAll("", typeof(SymbolDefinition));
        symbols = (SymbolDefinition[])Resources.FindObjectsOfTypeAll(typeof(SymbolDefinition));

        symbolMap = new Dictionary<Category, List<SymbolDefinition>>();

        var values = System.Enum.GetValues(typeof(Category));

        foreach (Category val in values)
        {
            if(val != Category.EMPTY)
                symbolMap[val] = new List<SymbolDefinition>();
        }

        foreach(var symbol in symbols)
        {
            if (symbol.category != Category.EMPTY)
                symbolMap[symbol.category].Add(symbol);
        }

        foreach (Category val in values)
        {
            if (val != Category.EMPTY)
                Debug.Log(val + " " +symbolMap[val].Count + " " + (int)val);
        }

        /*     SSelectionInfo info = new SSelectionInfo();
             info.grid = new ESelection[,] { { ESelection.None, ESelection.Right, ESelection.Wrong }, { ESelection.None, ESelection.Right, ESelection.Wrong }, { ESelection.None, ESelection.Right, ESelection.Wrong } };
             gridManager.OnSelectionStateChanged(info);*/

        /*    empty = Resources.Load<SymbolDefinition>("Images/Empty");
            smile = Resources.Load<SymbolDefinition>("Images/TestImg");*/
    }
	
	// Update is called once per frame
	void Update () {
    /*    selectionTimeLeft -= Time.deltaTime;
        if (selectionTimeLeft < 0)
        {
            selectionTimeLeft = 2.0f;

            SSelectionInfo info = new SSelectionInfo();
            info.grid = new ESelection[,] { { (ESelection)Random.Range(0, 3), (ESelection)Random.Range(0, 3), (ESelection)Random.Range(0, 3) },
                                            { (ESelection)Random.Range(0, 3), (ESelection)Random.Range(0, 3), (ESelection)Random.Range(0, 3) },
                                            { (ESelection)Random.Range(0, 3), (ESelection)Random.Range(0, 3), (ESelection)Random.Range(0, 3) } };
            gridManager.OnSelectionStateChanged(info);


            SPuzzleInfo puzzle = new SPuzzleInfo();
            puzzle.images = new Sprite[,] { { smile.image, empty.image, empty.image },
                                            { empty.image, smile.image, empty.image },
                                            { empty.image, empty.image, smile.image } };
            gridManager.DisplayNewPuzzle(puzzle);
        }


   /*     levelTimeLeft -= Time.deltaTime;
        if (levelTimeLeft < 0)
        {
            levelTimeLeft = 10.0f;

            SPuzzleInfo puzzle = new SPuzzleInfo();
            puzzle.images = new Sprite[,] { { smile.image, empty.image, empty.image },
                                            { empty.image, smile.image, empty.image },
                                            { empty.image, empty.image, smile.image } };
            gridManager.DisplayNewPuzzle(puzzle);
        }*/
    }
}
