﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;


public enum ESelection
{
	None,
	Wrong,
	Right
};

public class PuzzleInfo
{
	public Sprite[,] images;
}

public class SSelectionInfo
{
	public ESelection[,] grid;

	public override string ToString() 
	{
		string str = "";

		for (int y = 0; y < grid.GetLongLength(1); ++y)
		{
			for (int x = 0; x < grid.GetLongLength(0); ++x)
			{
				str += grid[y, x];
			}
		}
		return str;
	}
};

public enum EUIState
{
	Grid,
	Flippers,
	Transition
};

public enum EGridState
{
	Visible,
	TransitionToHidden,
	Hidden,
	TransitionToVisible,
};

public class GridImagesMgr : MonoBehaviour
{
	public Image TLImage = null;
	public Image TImage = null;
	public Image TRImage = null;
	public Image LImage = null;
	public Image CImage = null;
	public Image RImage = null;
	public Image BLImage = null;
	public Image BImage = null;
	public Image BRImage = null;

	public Image TLFrame = null;
	public Image TFrame = null;
	public Image TRFrame = null;
	public Image LFrame = null;
	public Image CFrame = null;
	public Image RFrame = null;
	public Image BLFrame = null;
	public Image BFrame = null;
	public Image BRFrame = null;

	List<Image> Images = new List<Image>();
	List<Image> Frames = new List<Image>();

	public Color NeutralColor;
	public Color CorrectColor;
	public Color WrongColor;
	public Color UnknownSelectionColor;

	public Sprite TestSprite;

	public AudioSource LaczankaMusic;
	public AudioSource PinballMusic;

    public event EventHandler OnFadeToGridFinished;

    EUIState State;

	EGridState GridState = EGridState.Hidden;

	PuzzleInfo QueuedPuzzleInfo = null;

	void Awake()
	{
		Images.Add(BLImage);
		Images.Add(BImage);
		Images.Add(BRImage);
		Images.Add(LImage);
		Images.Add(CImage);
		Images.Add(RImage);
		Images.Add(TLImage);
		Images.Add(TImage);
		Images.Add(TRImage);

		Frames.Add(BLFrame);
		Frames.Add(BFrame);
		Frames.Add(BRFrame);
		Frames.Add(LFrame);
		Frames.Add(CFrame);
		Frames.Add(RFrame);
		Frames.Add(TLFrame);
		Frames.Add(TFrame);
		Frames.Add(TRFrame);

		AppSettings.Instance.OnScreenConfigChanged += new ScreenConfigDelegate(this.OnScreenConfigChanged);
	}

	private void OnScreenConfigChanged(EScreenConfig new_config)
	{
		Canvas canvas = GetComponent<Canvas>();
		canvas.targetDisplay = (int)new_config;
	}

	// Use this for initialization
	void Start ()
	{
		PuzzleInfo puzzle_info = new PuzzleInfo();
		puzzle_info.images = new Sprite[,] { { TestSprite, TestSprite, TestSprite }, { TestSprite, TestSprite, TestSprite }, { TestSprite, TestSprite, TestSprite } };
		//DisplayNewPuzzle(puzzle_info);
	}
	
	// Update is called once per frame
	void Update ()
	{
		//if (Input.GetKeyDown(KeyCode.N))
		//{
		//	PuzzleInfo puzzle_info = new PuzzleInfo();
		//	puzzle_info.images = new Sprite[,] { { TestSprite, TestSprite, TestSprite }, { TestSprite, TestSprite, TestSprite }, { TestSprite, TestSprite, TestSprite } };
		//	DisplayNewPuzzle(puzzle_info);
		//}
		//if (Input.GetKeyDown(KeyCode.M))
		//{
		//	PlayHideAnimation();
		//}
		if (Input.GetKeyDown(KeyCode.K))
		{
			PlayTransitionToFlippersAnimation();
		}
		if(Input.GetKeyDown(KeyCode.L))
		{
			PlayTransitionToGridAnimation();
		}
	}

	public void DisplayNewPuzzle(PuzzleInfo puzzle_info)
	{
		if (State != EUIState.Grid)
		{
			SetImages(puzzle_info);
			GridState = EGridState.Hidden;
			PlayShowAnimation();
			return;
		}

		if(GridState == EGridState.Hidden)
		{
			SetImages(puzzle_info);
			PlayShowAnimation();
		}
		else if(GridState == EGridState.Visible)
		{
			QueuedPuzzleInfo = puzzle_info;
			PlayHideAnimation();
		}
		else if(GridState == EGridState.TransitionToVisible)
		{
			SetImages(puzzle_info);
		}
		else if(GridState == EGridState.TransitionToHidden)
		{
			QueuedPuzzleInfo = puzzle_info;
		}

		//if(GridState == EGridState.Visible)
		//{
		//	HideAll();
		//}

		//if (GridState != EGridState.Hidden && GridState != EGridState.Visible)
		//{
		//	QueuedPuzzleInfo = puzzle_info;
		//	return;
		//}

		
	}

	void SetImages(PuzzleInfo puzzle_info)
	{
		for (int y = 0; y < puzzle_info.images.GetLongLength(1); ++y)
		{
			for (int x = 0; x < puzzle_info.images.GetLongLength(0); ++x)
			{
				Image image = Images[y * 3 + x];
				image.sprite = puzzle_info.images[x, y];
			}
		}
	}

	public void OnSelectionStateChanged(SSelectionInfo selection_info)
	{
		Debug.Log("Selection state changed: " + selection_info);

		for (int y=0; y<selection_info.grid.GetLongLength(1); ++y)
		{
			for(int x=0; x<selection_info.grid.GetLongLength(0); ++x)
			{
				ESelection value = selection_info.grid[y, x];
				Image frame = Frames[y * 3 + x];
				switch(value)
				{
					case ESelection.None:
						frame.color = NeutralColor;
						break;

					case ESelection.Right:
						frame.color = ControlPanelScript.Instance.IsHintEnabled ? CorrectColor : UnknownSelectionColor;
						break;

					case ESelection.Wrong:
						frame.color = ControlPanelScript.Instance.IsHintEnabled ? WrongColor : UnknownSelectionColor;
						break;
				}
			}
		}
	}

	public void OnStartGame()
	{
		GetComponent<Animator>().SetTrigger("StartGame");
	}

	public void GoToFlippers()
	{
		PlayTransitionToFlippersAnimation();
	}

	public void PlayShowAnimation()
	{
		GridState = EGridState.TransitionToVisible;

		foreach (Image frame in Frames)
		{
			frame.GetComponent<Animator>().SetTrigger("Show");
		}
	}

	public void ShowAnimationFinished()
	{
		GridState = EGridState.Visible;
	}

	public void PlayHideAnimation()
	{
		if (State != EUIState.Grid)
			return;

		if (GridState != EGridState.Visible)
			return;

		GridState = EGridState.TransitionToHidden;
		foreach (Image frame in Frames)
		{
			frame.GetComponent<Animator>().SetTrigger("Hide");
		}
	}

	public void HideAnimationFinished()
	{
		GridState = EGridState.Hidden;
		if (QueuedPuzzleInfo != null)
		{
			DisplayNewPuzzle(QueuedPuzzleInfo);
			QueuedPuzzleInfo = null;
		}
	}

	public void PlayTransitionToFlippersAnimation()
	{
		if (State == EUIState.Grid)
		{
			GetComponent<Animator>().SetTrigger("GoToFlippers");
			State = EUIState.Transition;
		}
	}

	public void PlayTransitionToGridAnimation()
	{
		if (State == EUIState.Flippers)
		{ 
			GetComponent<Animator>().SetTrigger("GoToGrid");
			State = EUIState.Transition;
			InputAdapter.Instance.HackCleanInputs();
			InputAdapter.Instance.ForceInputRefresh();
		}
	}

	public void HideAll()
	{
		foreach(Image frame in Frames)
		{
			Color c = frame.color;
			c.a = 0;
			frame.color = c;
		}

		foreach (Image image in Images)
		{
			Color c = image.color;
			c.a = 0;
			image.color = c;
		}
	}

	public void FadeToFlippersHalfway()
	{
		GetComponent<Animator>().ResetTrigger("GoToGrid");
		Ball.paused = false;
	}

	public void FadeToGridHalfway()
	{
		GetComponent<Animator>().ResetTrigger("GoToFlippers");
		PlayShowAnimation();
		Ball.paused = true;
	}

	public void FadeToFlippersFinished()
	{
		Debug.Log("FadeToFlippersFinished");
		State = EUIState.Flippers;
	}

	public void FadeToGridFinished()
	{
		Debug.Log("FadeToGridFinished");
		State = EUIState.Grid;
        OnFadeToGridFinished(this, null);
	}
}
